package gs.smolban.model

import cats.Show
import java.time.Instant

/** Represents a smolban "tag", or arbitrary string descriptor.
  *
  * @param id
  *   The unique ID for this tag.
  * @param value
  *   The unique value of this tag.
  * @param createdAt
  *   The instant this tag was created.
  */
case class Tag(
  id: Tag.Id,
  value: Tag.Value,
  createdAt: Instant
)

object Tag:

  given CanEqual[Tag, Tag] = CanEqual.derived

  /** Opaque type which represents a [[Tag]] unique identifier. This is a long
    * integer.
    */
  opaque type Id = Long

  object Id:
    def apply(id: Long): Id = id

    given CanEqual[Id, Id] = CanEqual.derived

    given Show[Id] = t => t.toString()

  end Id

  /** Opaque type which represents a [[Tag]] value. These values are non-empty,
    * arbitrary strings.
    */
  opaque type Value = String

  object Value:

    /** Validate the candidate string - Tag Values must be non-empty.
      *
      * @param candidate
      *   The candidate string.
      * @return
      *   The valid value, or `None` if an empty string was given.
      */
    def validate(candidate: String): Option[Value] =
      if candidate.isEmpty() then None else Some(candidate)

    given CanEqual[Value, Value] = CanEqual.derived

    given Show[Value] = t => t

  end Value

end Tag
